/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "lin.h"

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void communication_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void lin_cfg_setup ( lin_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->rx_pin = HAL_PIN_NC;
    cfg->tx_pin = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->en   = HAL_PIN_NC;
    cfg->wk = HAL_PIN_NC;
    cfg->inh = HAL_PIN_NC;

    cfg->baud_rate  = 9600;
    cfg->data_bit   = UART_DEFAULT_DATA_BITS;
    cfg->parity_bit = UART_DEFAULT_PARITY_BIT;
    cfg->stop_bit   = UART_DEFAULT_STOP_BIT;
}

LIN_RETVAL lin_init ( lin_t *ctx, lin_cfg_t *cfg )
{
    uart_config_t uart_cfg;

    uart_configure_default( &uart_cfg );
    uart_cfg.pins.tx_pin = cfg->tx_pin;
    uart_cfg.pins.rx_pin = cfg->rx_pin;
    uart_cfg.baud        = cfg->baud_rate;
    uart_cfg.data_bits   = cfg->data_bit;
    uart_cfg.parity      = cfg->parity_bit;
    uart_cfg.stop        = cfg->stop_bit;

    if ( uart_open( &ctx->uart, &uart_cfg ) != UART_SUCCESS )
    {
        return LIN_INIT_ERROR;
    }

    // Output pins 

    digital_out_init( &ctx->en, cfg->en );
    digital_out_init( &ctx->wk, cfg->wk );
    digital_out_init( &ctx->inh, cfg->inh );

    return LIN_OK;

}

void lin_generic_multi_write ( lin_t *ctx, lin_data_t *data_buf,  uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;
    uart_write_block( &ctx->uart, &block);
}

void lin_generic_multi_read ( lin_t *ctx, lin_data_t *data_buf,  uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;

    uart_read_block( &ctx->uart, &block);
}

lin_data_t lin_generic_single_read ( lin_t *ctx )
{
    lin_data_t rx_data;

    rx_data =  uart_read( &ctx->uart );
    
    return rx_data;
}

void lin_generic_single_write ( lin_t *ctx, lin_data_t tx_data )
{
    uart_write( &ctx->uart, tx_data );
}

uint8_t lin_enable_pin_set( lin_t *ctx, uint8_t state )
{
    if ( state == LIN_PIN_STATE_HIGH )
    {
        digital_out_high( &ctx->en );
        return LIN_OK;
    }
    else if ( state == LIN_PIN_STATE_LOW )
    {
        digital_out_low( &ctx->en );
        return LIN_OK;
    }
    else
    {
        return LIN_FUNCTION_ERROR;
    }
}

uint8_t lin_inhibit_pin_set( lin_t *ctx, uint8_t state )
{
    if ( state == LIN_PIN_STATE_HIGH )
    {
        digital_out_high( &ctx->inh );
        return LIN_OK;
    }
    else if ( state == LIN_PIN_STATE_LOW )
    {
        digital_out_low( &ctx->inh );
        return LIN_OK;
    }
    else
    {
        return LIN_FUNCTION_ERROR;
    }
}

uint8_t lin_wake_pin_set( lin_t *ctx, uint8_t state )
{
    if ( state == LIN_PIN_STATE_HIGH )
    {
        digital_out_high( &ctx->wk );
        return LIN_OK;
    }
    else if ( state == LIN_PIN_STATE_LOW )
    {
        digital_out_low( &ctx->wk );
        return LIN_OK;
    }
    else
    {
        return LIN_FUNCTION_ERROR;
    }
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void communication_delay ( void )
{
    Delay_10ms(  );
}

// ------------------------------------------------------------------------- END

